#ifndef auxiliaryAxis_H
#define auxiliaryAxis_H
#include "turboPmacAxis.h"

// Forward declaration of the controller class to resolve the cyclic dependency
// between the controller and the axis .h-file. See
// https://en.cppreference.com/w/cpp/language/class.
class detectorTowerController;
class detectorTowerAxis;

class auxiliaryAxis : public turboPmacAxis {
  public:
    /**
     * @brief Construct a new detectorTowerAxis
     *
     * @param pController         Pointer to the associated controller
     * @param axisNo              Index of the axis
     */
    auxiliaryAxis(detectorTowerController *pController, int axisNo);

    /**
     * @brief Destroy the turboPmacAxis
     *
     */
    virtual ~auxiliaryAxis();

    /**
     * @brief Readout of some values from the controller at IOC startup
     *
     * @return asynStatus
     */
    asynStatus init();

    /**
     * @brief Implementation of the `stop` function from asynMotorAxis
     *
     * @param acceleration        Acceleration ACCEL from the motor record. This
     * value is currently not used.
     * @return asynStatus
     */
    asynStatus stop(double acceleration);

    /**
     * @brief Implementation of the `doPoll` function from sinqAxis. The
     * parameters are described in the documentation of `sinqAxis::doPoll`.
     *
     * @param moving
     * @return asynStatus
     */
    asynStatus doPoll(bool *moving);

    /**
     * @brief Set the target value for the detector angle and trigger a position
     * collection cycle
     *
     * @param position
     * @param relative
     * @param min_velocity
     * @param max_velocity
     * @param acceleration
     * @return asynStatus
     */
    asynStatus doMove(double position, int relative, double min_velocity,
                      double max_velocity, double acceleration);

    /**
     * @brief Call the reset function of the associated `detectorTowerAxis`.
     *
     * @return asynStatus
     */
    asynStatus reset();

  protected:
    detectorTowerController *pC_;
    detectorTowerAxis *dTA_;

  private:
    friend class detectorTowerAxis;
};

#endif
